﻿/**
 * @license Copyright © 2013 onwards, Andrew Whewell
 * All rights reserved.
 *
 * Redistribution and use of this software in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 *    * Neither the name of the author nor the names of the program's contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OF THE SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * @fileoverview A jQuery UI widget that displays a set of configuration shortcuts.
 */

(function(VRS, $, /** object= */ undefined)
{
    //region OptionShortcutPanelPluginState
    /**
     * The state object for the options shortcut panel.
     * @constructor
     */
    VRS.OptionShortcutPanelPluginState = function()
    {
        /**
         * A jQuery element for the container for the panel.
         * @type {jQuery=}
         */
        this.container = undefined;

        /**
         * A jQuery element for the parent list of shortcuts that will be shown.
         * @type {jQuery=}
         */
        this.listParent = undefined;

        /**
         * A jQuery element that shows the state of the auto-select settings.
         * @type {jQuery=}
         */
        this.autoSelectState = undefined;

        /**
         * The hook result for the auto-select changed event.
         * @type {object=}
         */
        this.autoSelectChangedHookResult = undefined;

        /**
         * The hook result for the locale changed event.
         * @type {object=}
         */
        this.localeChangedHookResult = undefined;
    };
    //endregion

    //region jQueryUIHelper
    VRS.jQueryUIHelper = VRS.jQueryUIHelper || {};
    VRS.jQueryUIHelper.getOptionShortcutPanelPlugin = function(jQueryElement) { return jQueryElement.data('vrsVrsOptionShortcutPanel'); };
    //endregion

    //region vrsOptionShortcutPanel
    /**
     * A jQuery UI plugin that displays shortcuts to some key options.
     */
    $.widget('vrs.vrsOptionShortcutPanel', {
        //region -- options
        options: {
            /** @type {bool} */                     showEnableDisableAutoSelect:    true,           // True if there should be some mechanism for enabling and disabling auto-select shown, false otherwise.
            /** @type {VRS.AircraftAutoSelect} */   aircraftAutoSelect:             undefined,      // The auto-select object to control - must be supplied if showEnableDisableAutoSelect is true.

            __nop: null
        },
        //endregion

        //region -- Widget methods - _create etc.
        /**
         * Returns the object carrying state for the widget, creating it if it does not already exist.
         * @returns {VRS.OptionShortcutPanelPluginState}
         * @private
         */
        _getState: function()
        {
            var result = this.element.data('optionShortcutPanelState');
            if(result === undefined) {
                result = new VRS.OptionShortcutPanelPluginState();
                this.element.data('optionShortcutPanelState', result);
            }

            return result;
        },

        /**
         * Creates the UI for the widget.
         * @private
         */
        _create: function()
        {
            var options = this.options;
            var state = this._getState();

            state.container =
                $('<div/>')
                    .addClass('vrsOptionShortcutPanel')
                    .appendTo(this.element);
            state.listParent =
                $('<ul/>')
                    .appendTo(state.container);

            state.localeChangedHookResult = VRS.globalisation.hookLocaleChanged(this._localeChanged, this);
            this._addEnableDisableAutoSelect(state, options);
        },

        /**
         * Releases resources allocated to the widget.
         * @private
         */
        _destroy: function()
        {
            var state = this._getState();
            var options = this.options;
            if(state.container) {
                this.element.empty();
                state.container = undefined;
            }
            if(state.listParent) state.listParent = undefined;
            if(state.autoSelectState) state.autoSelectState = undefined;

            if(state.localeChangedHookResult && VRS.globalisation) {
                VRS.globalisation.unhook(state.localeChangedHookResult);
                state.localeChangedHookResult = null;
            }

            if(state.autoSelectChangedHookResult && options && options.aircraftAutoSelect) {
                options.aircraftAutoSelect.unhook(state.autoSelectChangedHookResult);
                state.autoSelectChangedHookResult = undefined;
            }
        },
        //endregion

        //region -- Enable/disable auto-select
        /**
         * Adds the UI to allow the user to toggle the auto-select configuration switch.
         * @param state
         * @param options
         * @private
         */
        _addEnableDisableAutoSelect: function(state, options)
        {
            if(options.showEnableDisableAutoSelect) {
                if(!options.aircraftAutoSelect) throw 'You must supply the auto-select object that the shortcut panel will control';

                var autoSelect = options.aircraftAutoSelect;

                state.autoSelectState = $('<li/>')
                    .click(function() {
                        autoSelect.setEnabled(!autoSelect.getEnabled());
                        autoSelect.saveState();
                    })
                    .appendTo(state.listParent);
                this._showAutoSelectState();

                state.autoSelectChangedHookResult = autoSelect.hookEnabledChanged(this._showAutoSelectState, this);
            }
        },

        /**
         * Updates the UI to reflect the current setting of the auto-select toggle.
         * @private
         */
        _showAutoSelectState: function()
        {
            var state = this._getState();
            var autoSelect = this.options.aircraftAutoSelect;
            state.autoSelectState.text(autoSelect.getEnabled() ? VRS.$$.DisableAutoSelect : VRS.$$.EnableAutoSelect);
        },
        //endregion

        //region Events subscribed
        /**
         * Called when the user changes the language.
         * @private
         */
        _localeChanged: function()
        {
            this._showAutoSelectState();
        },
        //endregion

        nop: null
    });
    //endregion
}(window.VRS = window.VRS || {}, jQuery));
//endregion
